package net.weasel.HelpCenter;

import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.Scanner;

import org.bukkit.ChatColor;
import org.bukkit.event.Event;
import org.bukkit.event.Event.Priority;
import org.bukkit.plugin.PluginDescriptionFile;
import org.bukkit.plugin.java.JavaPlugin;
import org.bukkit.plugin.PluginManager;
import java.util.Calendar;
import java.text.SimpleDateFormat;
import java.net.*;

public class HelpCenter extends JavaPlugin 
{
	public static boolean hasWelcomeFile = false;
	
	private final HelpCenterListener playerListener = new HelpCenterListener(this);
	public static final String DATE_FMT = "yyyy-MM-dd HH:mm:ss";
	
	@Override
	public void onDisable() 
	{
		logOutput("HelpCenter disabled.");
	}

	@Override
	public void onEnable() 
	{
		initHelpCenter();
	}

	public static String cColour( String message )
	{
		CharSequence cChk = null;
		String temp = null;
		String[] Colours = { "#!k", "#!b", "#!g", "#!c", "#!r", "#!m", "#!y", "#!w", 
							 "#!K", "#!B", "#!G", "#!C", "#!R", "#!M", "#!Y", "#!W" };
		
		ChatColor[] cCode = { ChatColor.BLACK, ChatColor.DARK_BLUE, ChatColor.DARK_GREEN,
				           ChatColor.DARK_AQUA, ChatColor.DARK_RED, ChatColor.DARK_PURPLE,
				           ChatColor.GOLD, ChatColor.GRAY, ChatColor.DARK_GRAY,
				           ChatColor.BLUE, ChatColor.GREEN, ChatColor.AQUA, ChatColor.RED,
				           ChatColor.LIGHT_PURPLE, ChatColor.YELLOW, ChatColor.WHITE };
		
		for( int x = 0; x < Colours.length; x++ )
		{
			cChk = Colours[x];
			
			if( message.contains(cChk))
			{
				temp = message.replace(cChk, cCode[x].toString() );
				message = temp;
			}
			
		}
		
		return message;
		
	}
	
	public static class Dir implements FileFilter
	{
		private final String[] fileEx = new String[] { "txt" };
		
		static File[] getFiles(File path)
		{
			File files[];
			FileFilter filter = null;
			
			files = path.listFiles(filter);
		
			Arrays.sort(files);
			
			return( files );
		}

		@Override
		public boolean accept(File file) 
		{
			for( String extension : fileEx )
			{
				if( file.getName().toLowerCase().endsWith(extension))
				{
					return true;
				}
			}
			return false;
		}
	}
	
	public static void logOutput(String output)
	{
		System.out.println( oH() + output );
	}
	
	public static String oH()
	{
		Calendar cal = Calendar.getInstance();
		SimpleDateFormat sdf = new SimpleDateFormat(DATE_FMT);
		return( sdf.format(cal.getTime()) + " [INFO] [HelpCenter] ");
	}
	
	public void initHelpCenter()
	{
		PluginManager pm = getServer().getPluginManager();		
		pm.registerEvent(Event.Type.PLAYER_COMMAND, this.playerListener, Priority.Normal, this);
		pm.registerEvent(Event.Type.PLAYER_JOIN, this.playerListener, Priority.Normal,this );
		PluginDescriptionFile pdfFile = this.getDescription();
		
		File path = new File("HelpCenter");
		
		if( path.exists() == false )
		{
			logOutput("Directory 'HelpCenter' does not exist. Creating..");
			
			if( path.mkdir() == true )
			{
				logOutput("Created successfully.");
			}
			else
			{
				logOutput("Unable to create directory! Do you have write permission?");
			}
		}

		logOutput( pdfFile.getName() + " v" + pdfFile.getVersion() + " loaded.");
		
	}
	
	public static String getHelpFile(String who, String which) throws IOException
	{
		CharSequence badStr = "../";
		File hFile = null;
		
		if( which != "HCdefault" )
		{
			hFile = new File("HelpCenter/" + which.toLowerCase() + ".txt");
		}
		else
		{
			hFile = new File("HelpCenter/" + which + ".txt");
		}
		
		if( which.contains(badStr) == true )
		{
			logOutput( who + " tried to cheat the HelpCenter! : " + which.toString() );
			return( "No help found for " + which );
		}
		
		if( hFile.exists() == false )
		{
			return( "No help found for " + which );
		}
		else
		{
			byte[] buf = new byte[(int)hFile.length()];
			BufferedInputStream fr = null;
			
			try
			{
				fr = new BufferedInputStream(new FileInputStream(hFile.toString()));
				fr.read(buf);
			}
			finally
			{
				if( fr != null ) try { fr.close(); } catch (IOException ignored) { }
			}
			
			String result = new String(buf);
			
			if( result.substring(0,5).equals("[URL "))
			{
				String sub = result.substring(5);
				String[] spl = sub.split(" ]");
			 	String item = spl[0];
				return( fetchWebHelp(who, item) );
			 }
			else
			{
				return( cColour( new String(buf) ) );
			}
		}
	}
	
	public static String fetchWebHelp(String who, String address) throws MalformedURLException
	{
		logOutput( "Fetching help URL for " + who + ": " + address );
		String result = null;
		URLConnection fConn = null;
		
		try
		{
			fConn = new URL(address).openConnection();
			Scanner fSc = new Scanner(fConn.getInputStream());
			fSc.useDelimiter("\\Z");
			result = fSc.next();
		}
		catch (Exception e)
		{
			logOutput( "Unable to fetch web help URL: " + address );
			return( "Unable to fetch web help." );
		}
		
		return( result );
	}
}
